from jsonpath import jsonpath
from ddreport.exceptd import exceptContentObj
import openpyxl
import os


class PytestFunctions:

    # 抛出异常
    def __execptions(self, err):
        exceptContentObj.raiseException({"错误详情": err})

    # 错误数据
    def __raiseAssert(self, d1, d2, n=None):
        str_content = f"\n实际值{type(d1)}：{d1}\n预期值{type(d2)}：{d2}\n"
        index = f"行数：{n + 1}" if n is not None else ''
        err_str = str_content + index
        self.__execptions(err_str)

    def __raiseLoseData(self, d2):
        err_str = f"\n缺少数据为：{d2}"
        self.__execptions(err_str)

    # 字典字段显示与隐藏
    def jsonKeysShow(self, data, keys, is_show=True):
        def dictHandle(data, is_show):
            d = dict()
            is_show and [d.update({k: v}) for k, v in data.items() if k in keys] or [d.update({k: v}) for k, v in data.items() if k not in keys]
            return d
        if isinstance(data, dict):
            return dictHandle(data, is_show)
        elif isinstance(data, list):
            return [isinstance(i, dict) and dictHandle(i, is_show) or i for i in data]

    # json排序
    def jsonSort(self, data):
        def dictSort(data):
            d = dict(sorted(data.items(), key=lambda x: x[0]))
            return d
        if isinstance(data, dict):
            return dictSort(data)
        elif isinstance(data, list):
            return [isinstance(item, dict) and dictSort(item) or item for item in data]
        else:
            return data

    # 有序数据对比
    def checkOrder(self, data1, data2):
        """
        data1: 实际数据
        data2：预期数据
        """
        if data1 != data2:
            if isinstance(data1, (list, dict)):
                data1 = self.jsonSort(data1)
            if isinstance(data2, (list, dict)):
                data2 = self.jsonSort(data2)
            if isinstance(data1, list) and isinstance(data2, list):
                if len(data1) > len(data2):
                    for n, i in enumerate(data1):
                        if n > len(data2) - 1:
                            self.__raiseAssert(i, None, n)
                        elif i != data2[n]:
                            self.__raiseAssert(i, data2[n], n)
                else:
                    for n, i in enumerate(data2):
                        if n > len(data1) - 1:
                            self.__raiseAssert(None, i, n)
                        elif i != data1[n]:
                            self.__raiseAssert(data1[n], i, n)
            else:
                self.__raiseAssert(data1, data2)

    # 无序数据对比
    def uncheckOrder(self, data1, data2, only_keys=None):
        """
        data1: 实际数据
        data2：预期数据
        only_keys： 字段列表---一般选取唯一字段
        """
        if isinstance(data1, (list, dict)):
            data1 = self.jsonSort(data1)
        if isinstance(data2, (list, dict)):
            data2 = self.jsonSort(data2)
        if data1:
            data1_copy = data1.copy()
        if data2:
            data2_copy = data2.copy()
        if isinstance(data1, list) and isinstance(data2, list):
            for i in data1:
                data1_copy.remove(i)
                if not only_keys:
                    if i not in data2:
                        self.__raiseAssert(i, None)
                    else:
                        data2_copy.remove(i)
                else:
                    exp = ''
                    for k in only_keys:
                        s = isinstance(i[k], str) and f'"{i[k]}"' or f"{i[k]}"
                        exp += f'@.{k}=={s}' + ' and '
                    get_yq_data = jsonpath(data2, f'$..[?({exp[:-5]})]')
                    if not get_yq_data:
                        self.__raiseAssert(i, None)
                    else:
                        data2_copy.remove(get_yq_data[0])
                        self.checkOrder(i, get_yq_data[0])
            if data1_copy != data2_copy:
                # 丢失数据
                self.__raiseLoseData(data2_copy)
        else:
            if data1 != data2:
                self.__raiseAssert(data1, data2)

    # 读取xlsx并转为LIST_DICT
    def readXlsx(self, file_path, sheet_name='Sheet1', head=True) -> list:
        """文件路径，sheet页"""
        myxls = openpyxl.load_workbook(file_path)
        activeSheet = myxls[sheet_name]
        if head:
            keys, xlsxData = list(), list()
            for row in range(1, activeSheet.max_row + 1):
                d = dict()
                for n, column in enumerate(range(1, activeSheet.max_column + 1)):
                    data = activeSheet.cell(row=row, column=column).value
                    if len(keys) < activeSheet.max_column:
                        keys.append(data)
                    else:
                        d[keys[n]] = data
                if d:
                    xlsxData.append(d)
        else:
            xlsxData = list()
            for row in range(1, activeSheet.max_row + 1):
                xlsxData.append(
                    [activeSheet.cell(row=row, column=column).value for column in range(1, activeSheet.max_column + 1)])
        return xlsxData

    # 写入xlsx
    def writeXlsx(self, file_path, data):
        wb = openpyxl.Workbook()
        ws = wb.active
        for n, row in enumerate(data):
            if isinstance(row, dict):
                if n == 0:
                    ws.append(list(row.keys()))
                ws.append(list(row.values()))
            else:
                ws.append(row)
        wb.save(file_path)

    # 读取sql语句
    def readSql(self, sql_path, parameter=None):
        with open(sql_path, 'r', encoding='utf-8')as f:
            sql = f.read()
        if isinstance(parameter, dict):
            sql = sql.format(**parameter)
        return sql



import pymysql
from sshtunnel import SSHTunnelForwarder
from ddreport.exceptd import exceptContentObj


class PytestMysql:
    def __init__(self, db, db_ssh=None):
        self.__SSH = False
        if db_ssh:
            self.__SSH = True
            ssh_address_or_host, ssh_username = (db_ssh["host"], db_ssh['port']), db_ssh['user']
            remote_bind_address = (db['host'], db['port'])
            ssh_db_config = {"ssh_address_or_host": ssh_address_or_host, "ssh_username": ssh_username, "remote_bind_address": remote_bind_address}
            if db_ssh.get('password'):
                ssh_db_config['ssh_password'] = db_ssh.get('password')
            else:
                ssh_db_config['ssh_pkey'] = db_ssh.get('pkey') or None
            self.__server = SSHTunnelForwarder(**ssh_db_config)
        self.__condb(db)

    # 抛出异常
    def __execptions(self, err):
        exceptContentObj.raiseException({"错误详情": err})

    def __condb(self, db):
        try:
            data = {**db, **{'charset': 'utf8', 'cursorclass': pymysql.cursors.DictCursor, 'autocommit': True}}
            if self.__SSH is True:
                self.__server.start()
                data['host'], data['port'] = "127.0.0.1", self.__server.local_bind_port  # 重新赋值
            self.__conn = pymysql.connect(**data)
            self.__conn.ping(reconnect=True)
            self.__cursor = self.__conn.cursor()
        except Exception:
            self.__conn = None

    def query(self, selector):
        if self.__conn:
            self.__cursor.execute(selector)
            res = self.__cursor.fetchall()
            # self._colse()
            return res or []
        else:
            self.__execptions("Mysql Connection timed out: Connect failed")
            # return "Connection timed out: Connect failed"

    def off(self):
        if self.__conn:
            self.__cursor.close()  # 关闭游标
            self.__conn.close()  # 关闭数据库连接
            if self.__SSH is True:
                self.__server.close()


# db = {"host": "10.107.14.55", "port": 3306, "user": "root", "password": "123456"}
# db_ssh = {"host": "10.42.101.101", "port": 22, "user": "root", "password": None, "pkey": "kube_dev"}
#
# DB = DBMysql(db, db_ssh)
# a = DB.query("SET global sql_mode='STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,NO_ENGINE_SUBSTITUTION'")
# print(a)
#
# a1 = DB.query("select version()")
# print(a1)
#
# DB.db_colse()



class PytestErrorContent(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class ExceptContentObj:
    def __init__(self):
        self.value = None

    def raiseException(self, data):
        self.value = data
        pytestRaiseExcept()


def pytestRaiseExcept():
    raise PytestErrorContent(exceptContentObj)


exceptContentObj = ExceptContentObj()


#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: duanliangcong
# Mail: 137562703@qq.com
# Created Time:  2022-11-02 15:00:00
#############################################

# pip install twine
# python setup.py sdist
# twine upload dist/*

#############################################
#################使用方法#####################
#############################################
'''
目录结构
UPSDIST
    ddreport        库文件夹
    MANIFEST.in     配置
    setup.py        当前文件

1.cmd进入UPSDIST目录
2.执行命令：python setup.py sdist
3.执行命令：twine upload dist/*
'''



#### 每次更新需要修改 version 字段

from setuptools import setup, find_packages, find_namespace_packages

setup(
    name = "ddreport",
    version = "3.2",
    keywords = ("pip", "pytest","testReport"),
    description = "pytest测试报告",
    long_description = "优化错误详情的定位",
    license = "MIT Licence",

    url = "https://gitee.com/duanliangcong/dlc_pytest-report.git",
    author = "duanliangcong",
    author_email = "137562703@qq.com",
    entry_points={"pytest11": ["test_report=ddreport.testReport"]},

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = ["pytest", "requests", "sshtunnel", "pymysql", "jsonpath", 'openpyxl'],
)



#############################################
# File Name: setup.py
# Author: duanliangcong
# Mail: 137562703@qq.com
# Created Time:  2022-11-02 15:00:00
#############################################

from jsonpath import jsonpath
import requests
import pytest
import time
import json
import os, sys
import re
import ast
import traceback
from ddreport.api import PytestQuery
from ddreport.db import PytestMysql
from ddreport.func import PytestFunctions


requests.packages.urllib3.disable_warnings()

test_result = {
    "title": "",
    "desc": "",
    "tester": "",
    "start_time": "",
    "end_time": "",
    "failed": 0,
    "passed": 0,
    "skipped": 0,
    "error": 0,
    "cases": [],
}

dict_data = {
    "project": '',
    "model_sub": '0'
}


def getEnvData(env_path, env_name):
    try:
        with open(env_path, 'r', encoding='utf-8')as f:
            envs = json.loads(f.read())
        return jsonpath(envs, f'$..[?(@.env_name=="{env_name}")]')[0]
    except Exception:
        return dict()


def node_handle(node, item, call):
    d = dict()
    # 模块
    model_path = item.location[0].replace('.py', '').replace('\\', '/')
    d['model'] = dict_data['project'] + '/'.join(model_path.split('/')[int(dict_data['model_sub']):])
    # 类
    cf = item.location[-1].split('.')
    d['classed'] = None if len(cf) == 1 else cf[0]
    # 方法
    d['method'] = item.originalname
    # 描述
    d['doc'] = item.function.__doc__
    # 响应时间
    d['duration'] = str(call.duration)[:4]
    # 结果
    d['status'] = node.outcome
    # 详细内容
    if node.sections:
        if '<!DOCTYPE html>' in node.sections[0][-1]:
            d["print"] = node.sections[0][-1].replace('<', '&lt;').replace('>', '&gt;')
        else:
            d["print"] = node.sections[0][-1]
    if call.excinfo:
        try:
            excinfo = {"info": call.excinfo.value.value.value}
        except Exception:
            try:
                excinfo_str = re.findall(r'<ExceptionInfo.*?\("(.*?)"', str(call.excinfo))[0].replace('\\n', '\n')
            except Exception:
                excinfo_str = str(call.excinfo)
                sub_len = len(re.findall('tblen=\d+>', excinfo_str)[-1])
                excinfo_str = excinfo_str[:-sub_len]
                if excinfo_str.startswith('<'):
                    excinfo_str = excinfo_str[1:]
                if excinfo_str.endswith('>'):
                    excinfo_str = excinfo_str[:-1]
            try:
                if d['status'] != 'skipped':
                    _traceback = excinfo_str + '\n' + '\n'.join([str(i) for i in call.excinfo._traceback])
                else:
                    _traceback = excinfo_str.replace('ExceptionInfo ', '')
            except Exception:
                _traceback = '未知错误'
            excinfo = {"error": _traceback.replace('<', '&lt;').replace('>', '&gt;')}
        d.update(excinfo)
    return d


def pytest_addoption(parser):
    """添加main的可调用参数"""
    group = parser.getgroup("testreport")
    group.addoption(
        "--ddreport",
        action="store",
        default=None,
        help="测试报告标识",
    )
    group.addoption(
        "--title",
        action="store",
        default=None,
        help="测试报告最顶部标题",
    )
    group.addoption(
        "--desc",
        action="store",
        default=None,
        help="当前测试报告的说明",
    )
    group.addoption(
        "--tester",
        action="store",
        default=None,
        help="测试人员",
    )
    group.addoption(
        "--project",
        action="store",
        default=None,
        help="项目名称",
    )
    group.addoption(
        "--model_sub",
        action="store",
        default=None,
        help="测试模块显示的范围",
    ),
    group.addoption(
        "--env_path",
        action="store",
        default=None,
        help="环境配置路径",
    ),
    group.addoption(
        "--env_name",
        action="store",
        default=None,
        help="环境名称",
    ),
    group.addoption(
        "--receive_dict",
        action="store",
        default=None,
        help="接收一个字典类型的参数",
    ),


def pytest_sessionstart(session):
    test_result['start_time'] = time.strftime("%Y-%m-%d %H:%M:%S")
    test_result['title'] = session.config.getoption('--title') or ''
    test_result['tester'] = session.config.getoption('--tester') or ''
    test_result['desc'] = session.config.getoption('--desc') or ''
    dict_data['project'] = session.config.getoption('--project') or ''
    dict_data['model_sub'] = session.config.getoption('--model_sub') or '0'


def pytest_sessionfinish(session, exitstatus):
    def set_default(obj):
        if isinstance(obj, set):
            return f'{obj} ************************'
        # raise TypeError
    test_result['end_time'] = time.strftime("%Y-%m-%d %H:%M:%S")
    ddreport = session.config.getoption('--ddreport')
    if ddreport:
        report_dir = ddreport.replace('\\', '/')
        report_name = time.strftime("%Y-%m-%d_%H%M%S") + ' ' + 'Report.html'
        if not os.path.exists(report_dir):
            os.makedirs(report_dir)
        report_save_path = os.path.join(report_dir, report_name)
        # 读取测试报告文件
        template_path = os.path.join(os.path.dirname(__file__), './template')
        with open(f'{template_path}/index.html', 'r', encoding='utf-8')as f:
            template = f.read()
        report_template = template.replace('templateData', json.dumps(test_result, default=set_default))
        with open(report_save_path, 'w', encoding='utf-8') as f:
            f.write(report_template)


@pytest.hookimpl(tryfirst=True, hookwrapper=True)
def pytest_runtest_makereport(item, call):
    outcome = yield
    report = outcome.get_result()
    if report.when == 'call':
        test_result[report.outcome] += 1
        info = node_handle(report, item, call)
        test_result["cases"].append(info)
    elif report.outcome == 'failed':
        report.outcome = 'error'
        test_result['error'] += 1
        info = node_handle(report, item, call)
        test_result["cases"].append(info)
    elif report.outcome == 'skipped':
        test_result[report.outcome] += 1
        info = node_handle(report, item, call)
        test_result["cases"].append(info)


@pytest.fixture(scope='session')
def ddreport(request):
    # 环境获取
    env_path = request.config.getoption("--env_path")
    env_name = request.config.getoption("--env_name")
    get_env = getEnvData(env_path, env_name)
    # 全局变量
    receive_dict = request.config.getoption("--receive_dict")
    if receive_dict:
        try:
            receive_dict = json.loads(receive_dict)
        except Exception:
            receive_dict = ast.literal_eval(receive_dict)
        except Exception:
            receive_dict = None
    host = get_env.get('host') or ''
    db = get_env.get('db') or None
    db_ssh = get_env.get('db_ssh') or None
    return MyFixture(host, db, db_ssh, receive_dict)


class MyFixture:
    def __init__(self, host, _db, ssh_db, receive_dict):
        self.API = PytestQuery(host)
        self.DB = PytestMysql(_db, ssh_db)
        self.FC = PytestFunctions()
        self.GVALUE = dict()
        if isinstance(receive_dict, dict):
            self.GVALUE.update(receive_dict)
        super().__init__()


from ddreport.exceptd import exceptContentObj
from jsonpath import jsonpath
import requests
import ast
import traceback
import json
requests.packages.urllib3.disable_warnings()


class PytestQuery:
    def __init__(self, host=''):
        self.__HOST = host

    # 文件对象请求内容转str
    def __fileHandle(self, kwargs):
        if 'files' in kwargs.keys():
            kwargs_copy = kwargs.copy()
            kwargs_copy['files'] = str(kwargs_copy['files']).replace('<', "\"<").replace('>', ">\"")
            kwargs_copy['files'] = ast.literal_eval(kwargs_copy['files'])
            # self.__info.update({"请求内容": kwargs_copy})
            self.__info.update({"请求内容": json.dumps(kwargs_copy, indent=4, ensure_ascii=False)})
        else:
            # self.__info.update({"请求内容": kwargs})
            self.__info.update({"请求内容": json.dumps(kwargs, indent=4, ensure_ascii=False)})

    # 加载错误信息并抛出异常
    def __errdata(self, dict_data):
        if isinstance(dict_data, dict):
            self.__info.update(dict_data)
        self.__execptions()

    # 抛出异常
    def __execptions(self):
        exceptContentObj.raiseException(self.__info)

    # 接口请求
    def __pyQueryData(self, kwargs):
        self.__info = dict()
        if not kwargs['url'].startswith('http'):
            kwargs['url'] = self.__HOST + kwargs['url']
        self.__fileHandle(kwargs)
        try:
            r = requests.request(**kwargs)
        except Exception  as e:
            self.__errdata({"错误详情": f"{traceback.format_exc()}"})
        try:
            # response = r.json()
            response = json.dumps(r.json(), indent=4, ensure_ascii=False)
        except Exception:
            response = r.text.replace('<', '&lt;').replace('>', '&gt;')
        # 一个完整的信息
        # self.__info.update({"响应头": dict(r.headers), "响应Cookies": r.cookies.get_dict(), "响应体": response})
        self.__info.update({"响应头": json.dumps(dict(r.headers), indent=4, ensure_ascii=False), "响应Cookies": json.dumps(r.cookies.get_dict(), indent=4, ensure_ascii=False), "响应体": response})
        return r

    def __assertType(self, k_v):
        if k_v.upper() not in ["JSON", "TEXT"]:
            self.__errdata({"错误详情": f"断言异常，类型只能为JSON或TEXT"})

    # json断言数据类型判断
    def __assertParamsType(self, check):
        if not isinstance(check, (dict, list)):
            self.__errdata({"错误详情": f"断言异常，参数必须为json类型\n{check}"})
        if isinstance(check, dict):
            check = [check]
        return check

    # 响应的json数据类型判断
    def __assertDataType(self, r):
        try:
            response = r.json()
            return response
        except Exception:
            self.__errdata({"错误详情": f"响应体不是JSON类型"})

    # json数据匹配
    def __assertJsonCheck(self, data1, data2, n):
        if data1['value'] != data2:
            data1 = str(data1).replace('<', '&lt;').replace('>', '&gt;')
            self.__errdata({"失败详情": f"*** 断言匹配失败\n条数：{n + 1}\n匹配详情:{data1}"})

    # 文本数据匹配
    def __assertTextCheck(self, data1, data2, n):
        if data1['value'] not in data2:
            data1 = str(data1).replace('<', '&lt;').replace('>', '&gt;')
            self.__errdata({"失败详情": f"*** 断言匹配失败\n条数：{n + 1}\n匹配详情:{data1}"})

    # 断言主程序
    def __assertData(self, r, check):
        # 没有断言时的返回
        if not check:
            if r.status_code == 200:
                return r
            else:
                self.__errdata(None)
        # 断言验证
        else:
            check = self.__assertParamsType(check)
            for n, ass in enumerate(check):
                if ass['type'].upper() == 'JSON':
                    response = self.__assertDataType(r)
                    jsonpath_results = jsonpath(response, ass.get('exp')) or []
                    json_result = jsonpath_results[0] if len(jsonpath_results) == 1 else jsonpath_results
                    self.__assertJsonCheck(ass, json_result, n)
                elif ass['type'].upper() == 'TEXT':
                    if not isinstance(ass['value'], str):
                        ass['value'] = str(ass['value'])
                    self.__assertTextCheck(ass, r.text, n)
                else:
                    self.__assertType(ass['type'])
            return r

    def query(self, kwargs, check=None):
        r = self.__pyQueryData(kwargs)
        return self.__assertData(r, check)